﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace GlobalHookDemo
{
    class WordComparator : IComparer<WordEntry>
    {
        public int Compare(WordEntry w1, WordEntry w2)
        {
            if (w1.Priority != w2.Priority)
                return w2.Priority.CompareTo(w1.Priority);
            if (w1.UseCount != w2.UseCount)
                return w2.UseCount.CompareTo(w1.UseCount);
            return w1.Word.CompareTo(w2.Word);
        }
    }

    class DictionaryApi
    {
        public delegate void InitializeProgressBarDelegate();
        public event InitializeProgressBarDelegate InitializeProgressBarEvent;

        public delegate void ProgressBarMaximumDelegate(double max);
        public event ProgressBarMaximumDelegate ProgressBarMaximumEvent;

        public delegate void ProgressBarIncreaseDelegate();
        public event ProgressBarIncreaseDelegate ProgressBarIncreaseEvent;

        public delegate void FinishProgressBarDelegate();
        public event FinishProgressBarDelegate FinishProgressBarEvent;

        private static DictionaryApi instance;

        private DictionaryApi() { }

        public static DictionaryApi Instance
   {
      get 
      {
         if (instance == null)
         {
             instance = new DictionaryApi();
         }
         return instance;
      }
   }

        public SortedSet<WordEntry> words = new SortedSet<WordEntry>(new WordComparator()); 

        public IEnumerable<string> findWordsStartWith(string beginning)
        {
            
            var matchingList = new List<string>();
            beginning = beginning.ToLower().Trim();
            foreach (WordEntry word in words)
            {
                if (word.Word.ToLower().StartsWith(beginning.ToLower()))
                {
                    matchingList.Add(word.Word);
                }
            }
            return matchingList;
           
        }

        public void RemoveWord(WordEntry word)
        {
            words.Remove(word);
        }

        public void analyze(string text, bool withEvents = true)
        {
            if(withEvents) InitializeProgressBarEvent();
            string[] words = Regex.Split(text, "[\r\n ]");
            if (withEvents) ProgressBarMaximumEvent(words.Length);
            for (int i = 0; i < words.Length; i++)
            {
                if (withEvents) ProgressBarIncreaseEvent();
                addWord(words[i]);
            }
            if (withEvents) FinishProgressBarEvent();
        }

        public void readDictionary(string text)
        {
            string[] words = Regex.Split(text, "[\r\n ]");
            for (int i = 0; i < words.Length; i++)
            {
                readEntry(words[i]);
            }
        }


        public void readEntry(string entry)
        {
            entry = entry.Trim();
            if(entry.Length == 0)
                return;
            string[] splitted = entry.Split(',');
            try
            {
                int useCount = Convert.ToInt32(splitted[1]);
                int popularity = Convert.ToInt32(splitted[2]);
                words.Add(new WordEntry(splitted[0], useCount, popularity));
            }
            catch { }
        }

        public void updateDictionary()
        {
            var copy = words.ToArray();
            this.words = new SortedSet<WordEntry>(copy, new WordComparator()); 
        }

        public void addWord(string word)
        {
            //word = word.ToLower();
            string pattern = "[,.\"';/?()! ]+$";
            string replacement = "";
            Regex rgx = new Regex(pattern);
            word = rgx.Replace(word, replacement).Trim();
            if (word.Length == 0 || word.Equals("")) return;
            string[] splitted = word.Split(',');
            foreach (var foundWord in words)
            {
                if(foundWord.Word.Equals(word)) {
                    RemoveWord(foundWord);
                    foundWord.UseCount++;
                    words.Add(foundWord);
                    return;
                }
            } // not found
                words.Add(new WordEntry(word));
        }

        public void SaveDictionary()
        {
            System.IO.StreamWriter file = new System.IO.StreamWriter("slownik.txt");
            foreach (WordEntry w in DictionaryApi.Instance.words)
            {
                file.WriteLine(w.Word + "," + w.UseCount + "," + w.Priority);
            }
            file.Close();
        }

        internal IEnumerable<string> findMostPopular()
        {
            var ret = new List<string>();
            foreach (var word in words)
            {
                ret.Add(word.Word);
            }
            return ret;
        }
    }
}
