﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Windows;

namespace GlobalHookDemo
{

    class KeyboardEventArgs : EventArgs
    {
        public enum SpecialKey { None, Left, Right, Backspace, Delete };

        public KeyboardEventArgs()
        { }

        public KeyboardEventArgs(string _key)
        {
            this.specialKey = SpecialKey.None;
            this.key = _key;
        }

        public KeyboardEventArgs(SpecialKey _key)
        {
            this.specialKey = _key;
        }

        public SpecialKey specialKey { get; set; }
        public string key { get; set; }
    }
    
    class KeyboardHook
    {
        [DllImport("user32.dll")]
        static extern int ToUnicode(uint wVirtKey, uint wScanCode, byte[] lpKeyState,
           [Out, MarshalAs(UnmanagedType.LPWStr, SizeConst = 64)] StringBuilder pwszBuff, int cchBuff,
           uint wFlags);

        const uint KEYEVENTF_KEYUP = 0x0002;
        const uint KEYEVENTF_EXTENDEDKEY = 0x0001;

        private UserActivityHook hook;

        public string currentString {get; set;}
        private Boolean altKey, capsKey, shiftKey, ctrlKey = false;

        public delegate void KeyEntered(object sender, KeyboardEventArgs e);
        public event KeyEntered Entered;

        static KeyboardHook instance;

        public static KeyboardHook getInstance()
        {
            if (instance == null)
                instance = new KeyboardHook();
            return instance;
        }

        public void Stop()
        {
            hook.Stop();
        }

        public void Start()
        {
            hook.Start(false, true);
        }

        private KeyboardHook()
        {
            currentString = "";
            hook = new UserActivityHook(false, true);
            hook.KeyDown += new KeyEventHandler(MyKeyDown);
            hook.KeyUp += new KeyEventHandler(MyKeyUp);
        }

        public void MyKeyDown(object sender, KeyEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(e.KeyValue);
            switch (e.KeyValue)
            {
                case 32:
                    Entered(this, new KeyboardEventArgs(" "));
                    return;
                case 13:
                    Entered(this, new KeyboardEventArgs("\n"));
                    return;
                case 162:
                case 163:
                    ctrlKey = true;
                    return;
                case 164:
                case 165:
                    ctrlKey = false;
                    altKey = true;
                    return;
                case 160:
                case 161:
                    shiftKey = true;
                    return;
                case 20:
                    capsKey = !capsKey;
                    return;
                case 8:
                    Entered(this, new KeyboardEventArgs(KeyboardEventArgs.SpecialKey.Backspace));
                    return;
                case 37:
                    Entered(this, new KeyboardEventArgs(KeyboardEventArgs.SpecialKey.Left));
                    return;
                case 39:
                    Entered(this, new KeyboardEventArgs(KeyboardEventArgs.SpecialKey.Right));
                    return;
                case 46:
                    Entered(this, new KeyboardEventArgs(KeyboardEventArgs.SpecialKey.Delete));
                    return;
            }

            if (Properties.Settings.Default.SelectKey != null && Properties.Settings.Default.SelectKey.Equals(e.KeyData.ToString()))
            {
                Entered(this, new KeyboardEventArgs("selected"));
                e.Handled = true;
                return;
            }
            
            if (ctrlKey) return;
            byte[] keyStates = new byte[0x100];
            if ((!capsKey && shiftKey) || (capsKey && !shiftKey))
                keyStates[(int)Keys.ShiftKey] = 0xff;
            if (this.altKey)
            {
                keyStates[(int)Keys.ControlKey] = 0xff;
                keyStates[(int)Keys.Menu] = 0xff;
            }


            StringBuilder sb = new StringBuilder();
            int r = ToUnicode((uint)e.KeyValue, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, keyStates, sb, 10, 0);
            if (r >= 1)
            {
                string key = sb.ToString();
                Entered(this, new KeyboardEventArgs(key));
            }
        }


        public void MyKeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 165 || e.KeyValue == 164)
            {
                this.altKey = false;
                return;
            }
            if (e.KeyValue == 160 || e.KeyValue == 161)
            {
                shiftKey = false;
                return;
            }
            if (e.KeyValue == 162 || e.KeyValue == 163)
            {
                ctrlKey = false;
                return;
            }
        }
    }
}
