﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Forms;
using System.Drawing;

namespace GlobalHookDemo.Views
{
    /// <summary>
    /// Interaction logic for DictionaryControl.xaml
    /// </summary>
    public partial class DictionaryControl : System.Windows.Controls.UserControl
    {
        private string currentString = "";
        private int currentPosition = 0;

        public bool IsHooksEnabled { get; set; }

        public void update()
        {
            var words = DictionaryApi.Instance.findWordsStartWith(currentString);
            listBox1.ItemsSource = words.ToArray();
        }

        public DictionaryControl()
        {
            IsHooksEnabled = true;
            InitializeComponent();
            KeyboardHook.getInstance().Entered += new KeyboardHook.KeyEntered(DictionaryControl_Entered);
            listBox1.SelectionChanged += new SelectionChangedEventHandler(listBox1_SelectedIndexChanged);
            textBox1.TextChanged += new TextChangedEventHandler(textBox1_TextChanged);
            listBox1.FontSize = textBox1.FontSize = Properties.Settings.Default.FontSize;
            listBox1.FontFamily = textBox1.FontFamily = Properties.Settings.Default.FontFamily;
            listBox1.FontStretch = textBox1.FontStretch = Properties.Settings.Default.FontStretch;
            listBox1.FontStyle = textBox1.FontStyle = Properties.Settings.Default.FontStyle;
            listBox1.FontWeight = textBox1.FontWeight = Properties.Settings.Default.FontWeight;
            listBox1.ItemsSource = DictionaryApi.Instance.findWordsStartWith("");
        }



        void textBox1_TextChanged(object sender, EventArgs e)
        {
            if (!IsHooksEnabled)
                return;

            var words = DictionaryApi.Instance.findWordsStartWith(currentString);
            if (words.Count() == 0)
                words = DictionaryApi.Instance.findMostPopular();
            listBox1.ItemsSource = words.ToArray();
        }

        void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (!IsHooksEnabled)
                return;

            if (listBox1.SelectedItem == null)
                return;
            string selectedText = listBox1.SelectedItem.ToString();
            string ToAdd = currentString.Equals("") ? selectedText : selectedText.ToLower().Remove(0, currentString.ToLower().Length);
            ToAdd = ToAdd.TrimEnd();
            //DictionaryApi.Instance.addWord(currentString + ToAdd);
            KeyboardHook.getInstance().Stop();
            KeyboardWriter.SendKeys2(ToAdd);
            KeyboardHook.getInstance().Start();
            //SendKeys.SendWait(ToAdd);
            listBox1.SelectedIndex = -1;
            textBox1.Text = this.currentString = currentString + ToAdd;
            currentPosition = currentString.Length;
            textBox1.Focus();
        }

        void DictionaryControl_Entered(object sender, KeyboardEventArgs e)
        {
            if (!IsHooksEnabled)
                return;

            try
            {

                if (e.key != null)
                {
                    string txt = e.key;
                    if (String.Compare(txt, "selected") == 0)
                    {
                        listBox1.SelectedIndex = 0;
                        return;
                    }
                    currentString = currentString.Insert(currentPosition, txt);
                    currentPosition += txt.Length;
                    if (txt[0] == '\n' || txt[0] == ' ')
                    {
                        DictionaryApi.Instance.addWord(currentString);
                        this.currentString = "";
                        currentPosition = 0;
                    }
                }
                if (e.specialKey != KeyboardEventArgs.SpecialKey.None)
                {
                    switch (e.specialKey)
                    {
                        case KeyboardEventArgs.SpecialKey.Left:
                            currentPosition = Math.Max(0, currentPosition - 1);
                            break;
                        case KeyboardEventArgs.SpecialKey.Right:
                            currentPosition = Math.Min(currentString.Length, currentPosition + 1);
                            break;
                        case KeyboardEventArgs.SpecialKey.Backspace:
                            if (currentString.Length != 0 && currentPosition != 0)
                            {
                                currentString = currentString.Remove(currentPosition - 1, 1);
                                currentPosition -= 1;
                            }
                            break;
                        case KeyboardEventArgs.SpecialKey.Delete:
                            if (currentString.Length != 0 && currentPosition < currentString.Length)
                            {
                                currentString = currentString.Remove(currentPosition, 1);
                            }
                            break;
                    }
                }
                textBox1.Text = currentString;
            }
            catch
            { }
        }

    }
}
